﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class BackgroudQuadFader : MonoBehaviour {

    Mesh mesh;
	public MeshFilter meshFilter;
	/// <summary>
	/// True to move from left to right
	/// </summary>
	public bool LeftToRight = false;
    public Mesh faderMesh;
    Vector2[] UVsCurrent;
    Vector2[] UVsOriginal;
    Vector3[] VertOriginal;
    Vector3[] VertCurrent;

    public float PercentageProgression = 0;
    public float FadingSpeed = 0.1f;
    public float FadingOffSet = 1;
    
	// Use this for initialization
	void Start () {

		//PercentageProgression = 0;
		mesh = GetComponent<MeshFilter>().mesh;
        if (LeftToRight)
        {
            UVsOriginal = mesh.uv;
            UVsCurrent = new Vector2[UVsOriginal.Length];
            UVsCurrent[3] = UVsCurrent[1] = UVsOriginal[3];
            UVsCurrent[0] = UVsCurrent[2] = UVsOriginal[0];
            mesh.uv = UVsCurrent;

            VertOriginal = mesh.vertices;
            VertCurrent = new Vector3[VertOriginal.Length];
            VertCurrent[1] = VertCurrent[3] = VertOriginal[3];
            VertCurrent[2] = VertCurrent[0] = VertOriginal[0];
            mesh.vertices = VertCurrent;
        }else
        {
            UVsOriginal = mesh.uv;
            UVsCurrent = new Vector2[UVsOriginal.Length];
            UVsCurrent[3] = UVsCurrent[1] = UVsOriginal[1];
            UVsCurrent[0] = UVsCurrent[2] = UVsOriginal[2];
            mesh.uv = UVsCurrent;

            VertOriginal = mesh.vertices;
            VertCurrent = new Vector3[VertOriginal.Length];
            VertCurrent[3] = VertCurrent[1] = VertOriginal[1];
            VertCurrent[0] = VertCurrent[2] = VertOriginal[2];
            mesh.vertices = VertCurrent;
        }
        SetShiftingPercentage(PercentageProgression);
    }

    // Update is called once per frame
    void Update () {

        //PercentageProgression = Mathf.Clamp01(PercentageProgression + Time.deltaTime * FadingSpeed);
        //Debug.Log(PercentageProgression);
        //SetShiftingPercentage(PercentageProgression);

    }

    /// <summary>
    /// Function that set the Shift of the QuadFade
    /// to a specified percentage
    /// </summary>
    /// <param name="_percentage">Desired percentage of FadeShifting [between 0 and 1]</param>
    public void SetShiftingPercentage(float _percentage)
    {
        if (mesh != null)
        {
            if (!LeftToRight)
            {
                //Move the UV-Map till the parametrized _percentage level
                UVsCurrent[3] = new Vector2(1f - UVsOriginal[1].x * _percentage, UVsOriginal[3].y);
                UVsCurrent[0] = new Vector2(1f - UVsOriginal[2].x * _percentage, UVsOriginal[0].y);
                mesh.uv = UVsCurrent;

                VertCurrent[3] = new Vector3(((VertOriginal[3].x - VertOriginal[1].x) * _percentage) + VertOriginal[1].x, VertOriginal[3].y, 0);
                VertCurrent[0] = new Vector3(((VertOriginal[0].x - VertOriginal[2].x) * _percentage) + VertOriginal[2].x, VertOriginal[0].y, 0);
                mesh.vertices = VertCurrent;
            }
            else
            {
                //Move the UV-Map till the parametrized _percentage level
                UVsCurrent[1] = new Vector2(UVsOriginal[1].x * _percentage, UVsOriginal[1].y);
                UVsCurrent[2] = new Vector2(UVsOriginal[2].x * _percentage, UVsOriginal[2].y);
                mesh.uv = UVsCurrent;

                VertCurrent[1] = new Vector3(((VertOriginal[1].x - VertOriginal[3].x) * _percentage) + VertOriginal[3].x, VertOriginal[1].y, 0);
                VertCurrent[2] = new Vector3(((VertOriginal[2].x - VertOriginal[0].x) * _percentage) + VertOriginal[0].x, VertOriginal[2].y, 0);
                mesh.vertices = VertCurrent;
            }

            //OffSetFade(UVsCurrent, VertOriginal);
        }

    }

    void OffSetFade(Vector2[] _UVs, Vector3[] _Vertexes)
    {
        if (faderMesh != null)
        {

            Vector2[] UVsAlphas = new Vector2[_UVs.Length];
            Vector3[] VertAlphas = new Vector3[_Vertexes.Length];

            //SetUp UV-Map per il fader
            UVsAlphas[3] = _UVs[1];
            UVsAlphas[0] = _UVs[2];

            VertAlphas[3] = _Vertexes[1];
            VertAlphas[0] = _Vertexes[2];

            //Offset SetUp
            UVsAlphas[1] = new Vector2(_UVs[3].x + FadingOffSet, _UVs[3].y);
            UVsAlphas[0] = new Vector2(_UVs[2].x + FadingOffSet, _UVs[2].y);
            faderMesh.uv = UVsAlphas;

            VertAlphas[1] = new Vector3(_Vertexes[3].x + FadingOffSet, _Vertexes[3].y, _Vertexes[3].z);
            VertAlphas[0] = new Vector3(_Vertexes[2].x + FadingOffSet, _Vertexes[2].y, _Vertexes[2].z);
            faderMesh.vertices = VertAlphas;
        }
    }

}
