﻿using UnityEngine;
using System.Collections;

public class DoubleQuadGenerator : MonoBehaviour
{
    public Vector2[] polygon;
    void Start()
    {
        if (GetComponent<MeshRenderer>() == null) gameObject.AddComponent<MeshRenderer>();
        if (GetComponent<MeshFilter>() == null) gameObject.AddComponent<MeshFilter>();

        polygon = generateRandomPolygon(m_fWidth, m_fHeight);
        colliderToMesh(polygon);
    }

    private Vector2[] generateRandomPolygon(float fWidth, float fHeight)
    {
        Vector2[] vertices2D = new Vector2[6]; // 6 rectangle vertexes
        vertices2D[0] = new Vector2(-fWidth * .5f,-fHeight * 0.5f);
        vertices2D[1] = new Vector2(-fWidth * .5f, fHeight * 0.5f);
        vertices2D[2] = new Vector2( fWidth * .5f, -fHeight * 0.5f);
        vertices2D[3] = new Vector2( fWidth * .5f, fHeight * 0.5f);
        vertices2D[4] = new Vector2(fWidth * .250f, -fHeight * 0.5f);
        vertices2D[5] = new Vector2(fWidth * .251f, fHeight * 0.5f);

        return vertices2D;
    }

    private void normalize(float[] afVector)
    {
        float fNorm = 0;
        for (int i = 0; i < afVector.Length; i++)
        {
            fNorm += afVector[i] * afVector[i];
        }
        fNorm = Mathf.Sqrt(fNorm);
        for (int i = 0; i < afVector.Length; i++)
        {
            afVector[i] /= fNorm;
        }
    }

    private void colliderToMesh(Vector2[] polygon)
    {
        Mesh mesh = new Mesh();

        Vector3[] vertices = new Vector3[polygon.Length];
        for (int i = 0; i < polygon.Length; i++)
        {
            vertices[i] = new Vector3(polygon[i].x, polygon[i].y, 0);
        }
        mesh.vertices = vertices;

        Triangulator tr = new Triangulator(polygon);
        int[] triangles = tr.Triangulate();
        mesh.triangles = triangles;

        GetComponent<MeshFilter>().mesh = mesh;
        GetComponent<MeshRenderer>().material = shaderMat;
    }

#if UNITY_EDITOR
    private void OnDrawGizmos()
    {
        Gizmos.color = Color.blue;
        DrawGizmos();
    }

    private void OnDrawGizmosSelected()
    {
        Gizmos.color = Color.cyan;
        DrawGizmos();
    }

    void DrawGizmos()
    {
        Vector3 topLeft = transform.position + Vector3.left * m_fWidth * 0.5f + Vector3.up * m_fHeight;
        Vector3 topRight = transform.position + Vector3.right * m_fWidth / 2.0f + Vector3.up * m_fHeight;
        Vector3 bottomLeft = transform.position + Vector3.left * m_fWidth * 0.5f;
        Vector3 bottomRight = transform.position + Vector3.right * m_fWidth / 2.0f;

        Gizmos.DrawLine(topLeft,topRight);
        Gizmos.DrawLine(topLeft, bottomLeft);
        Gizmos.DrawLine(bottomLeft, bottomRight);
        Gizmos.DrawLine(bottomRight, topRight);
    }
#endif

    [SerializeField]
    private float m_fWidth = 1f;
    [SerializeField]
    private float m_fHeight = 1f;

    public Material shaderMat;
}
