﻿/*******************************************************************************

INTEL CORPORATION PROPRIETARY INFORMATION
This software is supplied under the terms of a license agreement or nondisclosure
agreement with Intel Corporation and may not be copied or disclosed except in
accordance with the terms of that agreement
Copyright(c) 2014 Intel Corporation. All Rights Reserved.

*******************************************************************************/

using UnityEngine;
using System.Collections;
using System;

public class FaceTracking : MonoBehaviour {

    public GameObject colorPlane; //Object to display color feed texture on
    public Texture2D colorTexture2D; //Color Texture
    private PXCMImage colorImage = null;//PXCMImage for color 

    private FaceRenderer faceRenderer; //Rendererer for Visualization
    private int MaxPoints = 78;
    private PXCMSenseManager psm = null; //SenseManager Instance
    private pxcmStatus sts; //Check error status
    private PXCMFaceModule faceAnalyzer; //FaceModule Instance

    public GameObject ManPlayer;
    public GameObject WomanPlayer;


    /// <summary>
    /// Use this for initialization
    /// Unity function called on the frame when a script is enabled 
    /// just before any of the Update methods is called the first time.
    /// </summary>
    void Start() {

        faceRenderer = gameObject.GetComponent<FaceRenderer>();
        wavingMgr[0] = ManPlayer.GetComponent<WavingManager>();
        wavingMgr[1] = WomanPlayer.GetComponent<WavingManager>();

        /* Initialize a PXCMSenseManager instance */
        psm = PXCMSenseManager.CreateInstance();
        if (psm == null)
        {
            Debug.Log("SenseManager Initialization Failed");
            return;
        }

        /* Enable the color stream of size 640x480 */
        psm.EnableStream(PXCMCapture.StreamType.STREAM_TYPE_COLOR, 1280, 720, 30);

        /* Enable the face tracking module*/
        sts = psm.EnableFace();
        if (sts != pxcmStatus.PXCM_STATUS_NO_ERROR) Debug.LogError("PXCSenseManager.EnableFace: " + sts);

        /* Retrieve an instance of face to configure */
        faceAnalyzer = psm.QueryFace();
        if (faceAnalyzer == null) Debug.LogError("PXCSenseManager.QueryFace");

        /* Initialize the execution pipeline */
        sts = psm.Init();
        if (sts != pxcmStatus.PXCM_STATUS_NO_ERROR)
        {
            Debug.LogError("PXCMSenseManager.Init Failed");
            OnDisable();
            return;
        }

        /* Retrieve a PXCMFaceConfiguration instance from a face to enable Gestures and Alerts */
        PXCMFaceConfiguration config = faceAnalyzer.CreateActiveConfiguration();
        config.detection.maxTrackedFaces = 2;
        config.detection.smoothingLevel = PXCMFaceConfiguration.SmoothingLevelType.SMOOTHING_MEDIUM;
        config.detection.isEnabled = true; // 3D detection is the default tracking mode.
        config.landmarks.isEnabled = true;
        config.pose.isEnabled = true;
        config.QueryExpressions().properties.maxTrackedFaces = 2;
        config.QueryExpressions().Enable();
        config.QueryExpressions().EnableExpression(PXCMFaceData.ExpressionsData.FaceExpression.EXPRESSION_MOUTH_OPEN);
        config.QueryExpressions().EnableExpression(PXCMFaceData.ExpressionsData.FaceExpression.EXPRESSION_KISS);
        config.EnableAllAlerts();
        config.ApplyChanges();
        config.Dispose();
    }

    int[] firstAcquisition = { -1, -1 };
    int[] plEmo = { 0, 0 };
    PXCMRectI32[] startRect = new PXCMRectI32[2];
    float[] deltaAngMax = { -1, -1};
    float[] destAngle = { -1, -1 };

    /// <summary>
    /// I WavingMgr sono switchati di volta in volta sulla base della posizione del rect (e quindi non in assoluto)
    /// </summary>
    WavingManager[] wavingMgr = new WavingManager[2];

    /// <summary>
    /// Update is called every frame, if the MonoBehaviour is enabled.
    /// </summary>
    void Update()
    {

        /* Make sure PXCMSenseManager Instance is Initialized */
        if (false)//psm != null)
        {

            /* Wait until any frame data is available true(aligned) false(unaligned) */
            if (psm.AcquireFrame(true) != pxcmStatus.PXCM_STATUS_NO_ERROR) return;

            /* Retrieve face tracking data if ready */
            faceAnalyzer = psm.QueryFace();
            if (faceAnalyzer != null)
            {
                PXCMFaceData _outputData = faceAnalyzer.CreateOutput();
                if (_outputData != null)
                {
                    _outputData.Update();

                    for (int i = 0; i < _outputData.QueryNumberOfDetectedFaces(); i++)
                    {
                        PXCMFaceData.Face _iFace = _outputData.QueryFaceByIndex(i);
                        if (_iFace != null)
                        {
                            /* Retrieve 78 Landmark Points */
                            PXCMFaceData.LandmarksData LandmarkData = _iFace.QueryLandmarks();
                            if (LandmarkData != null)
                            {
                                PXCMFaceData.LandmarkPoint[] landmarkPoints = new PXCMFaceData.LandmarkPoint[MaxPoints];
                                if (LandmarkData.QueryPoints(out landmarkPoints))
                                    faceRenderer.DisplayJoints2D(landmarkPoints);
                            }

                            /* Retrieve Detection Data */
                            PXCMFaceData.DetectionData detectionData = _iFace.QueryDetection();
                            if (detectionData != null)
                            {
                                PXCMRectI32 rect;
                                if (detectionData.QueryBoundingRect(out rect))
                                {
                                    if (i == 0)
                                        faceRenderer.SetDetectionRect1(rect);
                                    else
                                        faceRenderer.SetDetectionRect2(rect);

                                    {
                                        if (firstAcquisition[i] == -1)
                                        {
                                            firstAcquisition[i] = i;
                                            startRect[i] = rect;
                                        }
                                        else
                                        {
                                            Vector2 refVect = new Vector2(startRect[i].y, 0); refVect.Normalize();
                                            Vector2 destVect = new Vector2(rect.y, rect.x - startRect[i].x); destVect.Normalize();
                                            float dotVect = Vector2.Dot(refVect, destVect);
                                            float deltaAng = Mathf.Acos(dotVect) * Mathf.Rad2Deg;
                                            if (deltaAng > deltaAngMax[i])
                                                deltaAngMax[i] = deltaAng;
                                            destAngle[i] = Mathf.Clamp(-Mathf.Sign(destVect.y) * deltaAng / deltaAngMax[i], -1, 1);
                                        }
                                    }
                                }
                            }

                            /* Retrieve Pose Data */
                            //PXCMFaceData.PoseData poseData = _iFace.QueryPose();
                            //if (poseData != null)
                            //{
                            //    PXCMFaceData.PoseQuaternion poseQuaternion;
                            //    if (poseData.QueryPoseQuaternion(out poseQuaternion))
                            //        if (i == 0)
                            //            faceRenderer1.DisplayPoseQuaternion(poseQuaternion);
                            //        else
                            //            faceRenderer2.DisplayPoseQuaternion(poseQuaternion);
                            //}

                            /* Retrieve Expression Data */
                            PXCMFaceData.ExpressionsData expressionData = _iFace.QueryExpressions();
                            if (expressionData != null)
                            {
                                PXCMFaceData.ExpressionsData.FaceExpressionResult expressionResult;
                                if (expressionData.QueryExpression(PXCMFaceData.ExpressionsData.FaceExpression.EXPRESSION_MOUTH_OPEN, out expressionResult) &&
                                    expressionResult.intensity > 0.95f)
                                {
                                    plEmo[i] = 1;
                                }
                                else if (expressionData.QueryExpression(PXCMFaceData.ExpressionsData.FaceExpression.EXPRESSION_KISS, out expressionResult) &&
                                    expressionResult.intensity > 0.8f)
                                {
                                    plEmo[i] = 2;
                                }
                                else plEmo[i] = 0;
                            }

                        }
                    }

                    /* Retrieve Alert Data */
                    //PXCMFaceData.AlertData _alertData;
                    //for (int i = 0; i < _outputData.QueryFiredAlertsNumber(); i++)
                    //    if (_outputData.QueryFiredAlertData(i, out _alertData) == pxcmStatus.PXCM_STATUS_NO_ERROR)
                    //        faceRenderer.DisplayAlerts(_alertData);
                }
                _outputData.Dispose();
            }

             ///* Retrieve a sample from the camera */
            //PXCMCapture.Sample sample = psm.QueryFaceSample();
            //if (sample != null)
            //{
            //    colorImage = sample.color;
            //    if (colorImage != null)
            //    {
            //        if (colorTexture2D == null)
            //        {
            //            /* If not allocated, allocate a Texture2D */
            //            colorTexture2D = new Texture2D(colorImage.info.width, colorImage.info.height, TextureFormat.ARGB32, false);

            //            /* Associate the Texture2D with a gameObject */
            //            colorPlane.GetComponent<Renderer>().material.mainTexture = colorTexture2D;
            //            //colorPlane.renderer.material.mainTextureScale = new Vector2(-1f, 1f);
            //        }

            //        /* Retrieve the image data in Texture2D */
            //        PXCMImage.ImageData colorImageData;
            //        colorImage.AcquireAccess(PXCMImage.Access.ACCESS_READ, PXCMImage.PixelFormat.PIXEL_FORMAT_RGB32, out colorImageData);
            //        colorImageData.ToTexture2D(0, colorTexture2D);
            //        colorImage.ReleaseAccess(colorImageData);

            //        /* Apply the texture to the GameObject to display on */
            //        colorTexture2D.Apply();
            //    }
            //}

            /* Realease the frame to process the next frame */
            psm.ReleaseFrame();
        }
        else
        {
            PXCMRectI32 rect;
            rect.x =  rect.w = rect.h = 0;
            rect.y = 200;

            if (firstAcquisition[0] == -1)
            {
                firstAcquisition[0] = 0;
                startRect[0] = rect;
            }
            else
            {
                float axis = Input.GetAxisRaw("Horizontal2");
                rect.x = Convert.ToInt32(200f * Mathf.Asin(axis));
                rect.y = Convert.ToInt32(200f * Mathf.Acos(axis));
                rect.w = 10;
                rect.h = 10;

                Vector2 refVect = new Vector2(startRect[0].y, 0); refVect.Normalize();
                Vector2 destVect = new Vector2(rect.y, rect.x - startRect[0].x); destVect.Normalize();
                float dotVect = Vector2.Dot(refVect, destVect);
                float deltaAng = Mathf.Acos(dotVect) * Mathf.Rad2Deg;
                deltaAngMax[0] = 20;
                destAngle[0] = Mathf.Clamp(-Mathf.Sign(destVect.y) * deltaAng / deltaAngMax[0], -1, 1);
            }

            if (firstAcquisition[1] == -1)
            {
                firstAcquisition[1] = 0;
                startRect[1] = rect;
            }
            else
            {
                float axis = Input.GetAxisRaw("Horizontal1");
                rect.x = Convert.ToInt32(200f * Mathf.Asin(axis));
                rect.y = Convert.ToInt32(200f * Mathf.Acos(axis));
                rect.w = 10;
                rect.h = 10;

                Vector2 refVect = new Vector2(startRect[1].y, 0); refVect.Normalize();
                Vector2 destVect = new Vector2(rect.y, rect.x - startRect[1].x); destVect.Normalize();
                float dotVect = Vector2.Dot(refVect, destVect);
                float deltaAng = Mathf.Acos(dotVect) * Mathf.Rad2Deg;
                deltaAngMax[1] = 20;
                destAngle[1] = Mathf.Clamp(-Mathf.Sign(destVect.y) * deltaAng / deltaAngMax[1], -1, 1);
            }

        }

        if (firstAcquisition[0] != -1 && firstAcquisition[1] != -1)
        {
            if (startRect[0].x > startRect[1].x)
            {
                wavingMgr[0].WavePercentage = destAngle[0];
                wavingMgr[0].SetEmotion(plEmo[0]);
                wavingMgr[1].WavePercentage = destAngle[1];
                wavingMgr[1].SetEmotion(plEmo[1]);
            }
            else
            {
                wavingMgr[0].WavePercentage = destAngle[1];
                wavingMgr[0].SetEmotion(plEmo[1]);
                wavingMgr[1].WavePercentage = destAngle[0];
                wavingMgr[1].SetEmotion(plEmo[0]);
            }
        }
        else
        {
            wavingMgr[0].WavePercentage = destAngle[0];
            wavingMgr[1].WavePercentage = destAngle[0];
            wavingMgr[0].SetEmotion(plEmo[0]);
            wavingMgr[1].SetEmotion(plEmo[0]);
        }
    }

    /// <summary>
    /// Unity function that is called when the behaviour becomes disabled () or inactive.
    /// Used for clean-up in the end
    /// </summary>
    void OnDisable()
    {
        if(faceAnalyzer != null)
		faceAnalyzer.Dispose();
        if (psm != null)
        psm.Dispose();
    }
}
